@@ -1,6 +1,6 @@
 /* 
    Tests for high-level HTTP interface (ne_basic.h)
-   Copyright (C) 2002-2003, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 2002-2008, 2012, Joe Orton <joe@manyfish.co.uk>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -46,8 +46,10 @@ static int content_type(void)
 	{ "foo/bar", "foo", "bar", NULL },
 	{ "foo/bar  ", "foo", "bar", NULL },
 	{ "application/xml", "application", "xml", NULL },
-	/* text/ subtypes default to charset ISO-8859-1. */
+	/* text/ subtypes default to charset ISO-8859-1, per 2616. */
 	{ "text/lemon", "text", "lemon", "ISO-8859-1" },
+        /* text/xml defaults to charset us-ascii, per 3280 */
+        { "text/xml", "text", "xml", "us-ascii" },        
 #undef TXU
 #define TXU "text", "xml", "utf-8"
 	/* 2616 doesn't *say* that charset can be quoted, but bets are
@@ -69,10 +71,36 @@ static int content_type(void)
     };
 
     for (n = 0; ctypes[n].value != NULL; n++) {
-	ne_content_type ct = {0};
+        ne_content_type ct;
+        ne_session *sess;
+        ne_request *req;
+        char resp[200];
+        int rv;
+
+        ct.type = ct.subtype = ct.charset = ct.value = "unset";
+
+        ne_snprintf(resp, sizeof resp,
+                    "HTTP/1.0 200 OK\r\n" "Content-Length: 0\r\n"
+                    "Content-Type: %s\r\n" "\r\n", ctypes[n].value);
+        
+        CALL(make_session(&sess, single_serve_string, resp));
+
+        req = ne_request_create(sess, "GET", "/anyfoo");
+        ONREQ(ne_request_dispatch(req));
+        rv = ne_get_content_type(req, &ct);
+        
+        ONV(rv == 0 && !ctypes[n].type,
+            ("expected c-t parse failure for %s", ctypes[n].value));
+
+        ONV(rv != 0 && ctypes[n].type,
+            ("c-t parse failure %d for %s", rv, ctypes[n].value));
+
+        ne_request_destroy(req);
+        ne_session_destroy(sess);
+        CALL(await_server());
+
+        if (rv) continue;	
 
-	ne_content_type_handler(&ct, ctypes[n].value);
-	
 	ONV(strcmp(ct.type, ctypes[n].type),
 	    ("for `%s': type was `%s'", ctypes[n].value, ct.type));
 
@@ -90,7 +118,7 @@ static int content_type(void)
 	    strcmp(ctypes[n].charset, ct.charset),
 	    ("for `%s': charset was `%s'", ctypes[n].value, ct.charset));
 
-	NE_FREE(ct.value);
+	ne_free(ct.value);
     }
 
     return OK;
@@ -116,13 +144,14 @@ static int do_range(off_t start, off_t end, const char *fail,
     ret = ne_get_range(sess, "/foo", &range, fd);
 
     close(fd);
+    ne_close_connection(sess);
     CALL(await_server());
     
     if (fail) {
 #if 0
 	t_warning("error was %s", ne_get_error(sess));
 #endif
-	ONN(fail, ret == NE_OK);
+	ONV(ret == NE_OK, ("%s", fail));
     } else {
 	ONREQ(ret);
     }
@@ -135,7 +164,15 @@ static int get_range(void)
 {
     return do_range(1, 10, NULL,
 		    "HTTP/1.1 206 Widgets\r\n" "Connection: close\r\n"
-		    "Content-Range: bytes 1-10\r\n"
+		    "Content-Range: bytes 1-10/10\r\n"
+		    "Content-Length: 10\r\n\r\nabcdefghij");
+}
+
+static int get_eof_range(void)
+{
+    return do_range(1, -1, NULL,
+		    "HTTP/1.1 206 Widgets\r\n" "Connection: close\r\n"
+		    "Content-Range: bytes 1-10/10\r\n"
 		    "Content-Length: 10\r\n\r\nabcdefghij");
 }
 
@@ -143,7 +180,7 @@ static int fail_range_length(void)
 {
     return do_range(1, 10, "range response length mismatch should fail",
 		    "HTTP/1.1 206 Widgets\r\n" "Connection: close\r\n"
-		    "Content-Range: bytes 1-2\r\n"
+		    "Content-Range: bytes 1-2/2\r\n"
 		    "Content-Length: 2\r\n\r\nab");
 }
 
@@ -151,7 +188,7 @@ static int fail_range_units(void)
 {
     return do_range(1, 2, "range response units check should fail",
 		    "HTTP/1.1 206 Widgets\r\n" "Connection: close\r\n"
-		    "Content-Range: fish 1-2\r\n"
+		    "Content-Range: fish 1-2/2\r\n"
 		    "Content-Length: 2\r\n\r\nab");
 }
 
@@ -159,7 +196,7 @@ static int fail_range_notrange(void)
 {
     return do_range(1, 2, "non-ranged response should fail",
 		    "HTTP/1.1 200 Widgets\r\n" "Connection: close\r\n"
-		    "Content-Range: bytes 1-2\r\n"
+		    "Content-Range: bytes 1-2/2\r\n"
 		    "Content-Length: 2\r\n\r\nab");
 }
 
@@ -206,23 +243,86 @@ static int dav_capabilities(void)
 	ONV(c.dav_executable != caps[n].exec,
 	    ("class2 was %d not %d", c.dav_executable, caps[n].exec));
 
-	CALL(await_server());
-
-        ne_session_destroy(sess);
+	CALL(destroy_and_wait(sess));
     }
 
     return OK;	
 }
 
+static int get(void)
+{
+    ne_session *sess;
+    int fd;
+    
+    CALL(make_session(&sess, single_serve_string, 
+                      "HTTP/1.0 200 OK\r\n"
+                      "Content-Length: 5\r\n"
+                      "\r\n"
+                      "abcde"));
+    
+    fd = open("/dev/null", O_WRONLY);
+    ONREQ(ne_get(sess, "/getit", fd));
+    close(fd);
+
+    return destroy_and_wait(sess);
+}
+
+#define CLASS_12 (NE_CAP_DAV_CLASS1 | NE_CAP_DAV_CLASS2)
+
+static int options2(void)
+{
+    static const struct {
+        const char *hdrs;
+        unsigned int caps;
+    } ts[] = {
+        { "1,2\r\n", CLASS_12 },
+        { "1 2\r\n", 0 },
+        /* these aren't strictly legal headers: */
+        { "2,1\r\n", CLASS_12 },
+        { " 1, 2  \r\n", CLASS_12 },
+        { "1\r\nDAV:2\r\n", CLASS_12 },
+        /* extended types */
+        { "1, 2, extended-mkcol", CLASS_12 | NE_CAP_EXT_MKCOL },
+        { NULL, 0 }
+    };
+    char resp[BUFSIZ];
+    int n;
+
+    for (n = 0; ts[n].hdrs != NULL; n++) {
+        ne_session *sess;
+        unsigned int caps;
+
+        ne_snprintf(resp, BUFSIZ, "HTTP/1.0 200 OK\r\n"
+                    "Connection: close\r\n"
+                    "Content-Length: 0\r\n"
+                    "DAV: %s" "\r\n\r\n", ts[n].hdrs);
+
+        CALL(make_session(&sess, single_serve_string, resp));
+
+        ONREQ(ne_options2(sess, "/foo", &caps));
+
+        ONV(caps != ts[n].caps,
+            ("capabilities for 'DAV: %s' were 0x%x, expected 0x%x", 
+             ts[n].hdrs, caps, ts[n].caps));
+
+        CALL(destroy_and_wait(sess));
+    }
+
+    return OK;  
+}
+
 ne_test tests[] = {
     T(lookup_localhost),
     T(content_type),
     T(get_range),
+    T(get_eof_range),
     T(fail_range_length),
     T(fail_range_units),
     T(fail_range_notrange),
     T(fail_range_unsatify),
     T(dav_capabilities),
+    T(get),
+    T(options2),
     T(NULL) 
 };
 
